@@ -1,6 +1,6 @@
 /*
     FUSE: Filesystem in Userspace
-    Copyright (C) 2001-2004  Miklos Szeredi <miklos@szeredi.hu>
+    Copyright (C) 2001-2005  Miklos Szeredi <miklos@szeredi.hu>
 
     This program can be distributed under the terms of the GNU GPL.
     See the file COPYING.
@@ -40,30 +40,40 @@ static int hello_getdir(const char *path, fuse_dirh_t h, fuse_dirfil_t filler)
     if(strcmp(path, "/") != 0)
         return -ENOENT;
 
-    filler(h, ".", 0);
-    filler(h, "..", 0);
-    filler(h, hello_path + 1, 0);
+    filler(h, ".", 0, 0);
+    filler(h, "..", 0, 0);
+    filler(h, hello_path + 1, 0, 0);
 
     return 0;
 }
 
-static int hello_open(const char *path, int flags)
+static int hello_open(const char *path, struct fuse_file_info *fi)
 {
     if(strcmp(path, hello_path) != 0)
         return -ENOENT;
 
-    if((flags & 3) != O_RDONLY)
+    if((fi->flags & 3) != O_RDONLY)
         return -EACCES;
 
     return 0;
 }
 
-static int hello_read(const char *path, char *buf, size_t size, off_t offset)
+static int hello_read(const char *path, char *buf, size_t size, off_t offset,
+                      struct fuse_file_info *fi)
 {
+    size_t len;
+    (void) fi;
     if(strcmp(path, hello_path) != 0)
         return -ENOENT;
-    
-    memcpy(buf, hello_str + offset, size);
+
+    len = strlen(hello_str);
+    if (offset < len) {
+        if (offset + size > len)
+            size = len - offset;
+        memcpy(buf, hello_str + offset, size);
+    } else
+        size = 0;
+
     return size;
 }
 
@@ -76,6 +86,5 @@ static struct fuse_operations hello_oper = {
 
 int main(int argc, char *argv[])
 {
-    fuse_main(argc, argv, &hello_oper);
-    return 0;
+    return fuse_main(argc, argv, &hello_oper);
 }
