@@ -1,6 +1,6 @@
 /*
     FUSE: Filesystem in Userspace
-    Copyright (C) 2001  Miklos Szeredi (mszeredi@inf.bme.hu)
+    Copyright (C) 2001-2005  Miklos Szeredi <miklos@szeredi.hu>
 
     This program can be distributed under the terms of the GNU GPL.
     See the file COPYING.
@@ -18,7 +18,7 @@ static int null_getattr(const char *path, struct stat *stbuf)
 {
     if(strcmp(path, "/") != 0)
         return -ENOENT;
-    
+
     stbuf->st_mode = S_IFREG | 0644;
     stbuf->st_nlink = 1;
     stbuf->st_uid = getuid();
@@ -38,7 +38,7 @@ static int null_truncate(const char *path, off_t UNUSED(size))
     return 0;
 }
 
-static int null_open(const char *path, int UNUSED(flags))
+static int null_open(const char *path, struct fuse_file_info *UNUSED(fi))
 {
     if(strcmp(path, "/") != 0)
         return -ENOENT;
@@ -47,7 +47,7 @@ static int null_open(const char *path, int UNUSED(flags))
 }
 
 static int null_read(const char *path, char *UNUSED(buf), size_t size,
-                     off_t UNUSED(offset))
+                     off_t UNUSED(offset), struct fuse_file_info *UNUSED(fi))
 {
     if(strcmp(path, "/") != 0)
         return -ENOENT;
@@ -56,7 +56,7 @@ static int null_read(const char *path, char *UNUSED(buf), size_t size,
 }
 
 static int null_write(const char *path, const char *UNUSED(buf), size_t size,
-                     off_t UNUSED(offset))
+                     off_t UNUSED(offset), struct fuse_file_info *UNUSED(fi))
 {
     if(strcmp(path, "/") != 0)
         return -ENOENT;
@@ -64,23 +64,15 @@ static int null_write(const char *path, const char *UNUSED(buf), size_t size,
     return size;
 }
 
-static int null_statfs(struct fuse_statfs *st)
-{
-    return st->block_size = st->blocks = st->blocks_free = st->files =
-	st->files_free = st->namelen = 0;
-}
-
 static struct fuse_operations null_oper = {
     .getattr	= null_getattr,
     .truncate	= null_truncate,
     .open	= null_open,
     .read	= null_read,
     .write	= null_write,
-    .statfs	= null_statfs,
 };
 
 int main(int argc, char *argv[])
 {
-    fuse_main(argc, argv, &null_oper);
-    return 0;
+    return fuse_main(argc, argv, &null_oper);
 }
