@@ -1,6 +1,6 @@
 /*
     FUSE: Filesystem in Userspace
-    Copyright (C) 2001-2004  Miklos Szeredi <miklos@szeredi.hu>
+    Copyright (C) 2001-2005  Miklos Szeredi <miklos@szeredi.hu>
 
     This program can be distributed under the terms of the GNU LGPL.
     See the file COPYING.LIB.
@@ -12,6 +12,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <pthread.h>
+#include <unistd.h>
 #include <signal.h>
 #include <errno.h>
 #include <sys/time.h>
@@ -36,13 +37,14 @@ static void *do_work(void *data)
     struct fuse_worker *w = (struct fuse_worker *) data;
     struct fuse *f = w->f;
     struct fuse_context *ctx;
+    int is_mainthread = (f->numworker == 1);
 
     ctx = (struct fuse_context *) malloc(sizeof(struct fuse_context));
     if (ctx == NULL) {
         fprintf(stderr, "fuse: failed to allocate fuse context\n");
-        pthread_mutex_lock(&f->lock);
+        pthread_mutex_lock(&f->worker_lock);
         f->numavail --;
-        pthread_mutex_unlock(&f->lock);
+        pthread_mutex_unlock(&f->worker_lock);
         return NULL;
     }
     pthread_setspecific(context_key, ctx);
@@ -53,15 +55,15 @@ static void *do_work(void *data)
     while (1) {
         struct fuse_cmd *cmd;
 
-        if (__fuse_exited(f))
+        if (fuse_exited(f))
             break;
 
-        cmd = __fuse_read_cmd(w->f);
+        cmd = fuse_read_cmd(w->f);
         if (cmd == NULL)
             continue;
 
         if (f->numavail == 0 && f->numworker < FUSE_MAX_WORKERS) {
-            pthread_mutex_lock(&f->lock);
+            pthread_mutex_lock(&f->worker_lock);
             if (f->numworker < FUSE_MAX_WORKERS) {
                 /* FIXME: threads should be stored in a list instead
                    of an array */
@@ -69,20 +71,24 @@ static void *do_work(void *data)
                 pthread_t *thread_id = &w->threads[f->numworker];
                 f->numavail ++;
                 f->numworker ++;
-                pthread_mutex_unlock(&f->lock);
+                pthread_mutex_unlock(&f->worker_lock);
                 res = start_thread(w, thread_id);
                 if (res == -1) {
-                    pthread_mutex_lock(&f->lock);
+                    pthread_mutex_lock(&f->worker_lock);
                     f->numavail --;
-                    pthread_mutex_unlock(&f->lock);
+                    pthread_mutex_unlock(&f->worker_lock);
                 }
             } else
-                pthread_mutex_unlock(&f->lock);
+                pthread_mutex_unlock(&f->worker_lock);
         }
 
         w->proc(w->f, cmd, w->data);
     }
 
+    /* Wait for cancellation */
+    if (!is_mainthread)
+        pause();
+
     return NULL;
 }
 
@@ -101,7 +107,7 @@ static int start_thread(struct fuse_worker *w, pthread_t *thread_id)
         fprintf(stderr, "fuse: error creating thread: %s\n", strerror(res));
         return -1;
     }
-    
+
     pthread_detach(*thread_id);
     return 0;
 }
@@ -130,8 +136,8 @@ static int mt_create_context_key()
         if (err)
             fprintf(stderr, "fuse: failed to create thread specific key: %s\n",
                     strerror(err));
-        else 
-            __fuse_set_getcontext_func(mt_getcontext);
+        else
+            fuse_set_getcontext_func(mt_getcontext);
     }
     if (!err)
         context_ref ++;
@@ -144,18 +150,18 @@ static void mt_delete_context_key()
     pthread_mutex_lock(&context_lock);
     context_ref--;
     if (!context_ref) {
-        __fuse_set_getcontext_func(NULL);
+        fuse_set_getcontext_func(NULL);
         pthread_key_delete(context_key);
     }
     pthread_mutex_unlock(&context_lock);
 }
 
-int __fuse_loop_mt(struct fuse *f, fuse_processor_t proc, void *data)
+int fuse_loop_mt_proc(struct fuse *f, fuse_processor_t proc, void *data)
 {
     struct fuse_worker *w;
     int i;
 
-    w = malloc(sizeof(struct fuse_worker));    
+    w = malloc(sizeof(struct fuse_worker));
     if (w == NULL) {
         fprintf(stderr, "fuse: failed to allocate worker structure\n");
         return -1;
@@ -165,16 +171,6 @@ int __fuse_loop_mt(struct fuse *f, fuse_processor_t proc, void *data)
     w->data = data;
     w->proc = proc;
 
-#ifdef USE_UCLIBC
-    {
-      pthread_mutexattr_t attr;
-      pthread_mutexattr_init(&attr);
-      pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ADAPTIVE_NP);
-      pthread_mutex_init(&context_lock, &attr);
-      pthread_mutexattr_destroy(&attr);
-    }
-#endif
-
     if (mt_create_context_key() != 0) {
         free(w);
         return -1;
@@ -188,6 +184,7 @@ int __fuse_loop_mt(struct fuse *f, fuse_processor_t proc, void *data)
     pthread_mutex_unlock(&f->lock);
     mt_delete_context_key();
     free(w);
+    f->exited = 0;
     return 0;
 }
 
@@ -196,5 +193,7 @@ int fuse_loop_mt(struct fuse *f)
     if (f == NULL)
         return -1;
 
-    return __fuse_loop_mt(f, (fuse_processor_t) __fuse_process_cmd, NULL);
+    return fuse_loop_mt_proc(f, (fuse_processor_t) fuse_process_cmd, NULL);
 }
+
+__asm__(".symver fuse_loop_mt_proc,__fuse_loop_mt@");
