@@ -1,12 +1,13 @@
 /*
     FUSE: Filesystem in Userspace
-    Copyright (C) 2001-2004  Miklos Szeredi <miklos@szeredi.hu>
+    Copyright (C) 2001-2005  Miklos Szeredi <miklos@szeredi.hu>
 
     This program can be distributed under the terms of the GNU LGPL.
     See the file COPYING.LIB.
 */
 
-#include "fuse.h"
+#include "fuse_i.h"
+#include "fuse_compat.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -19,32 +20,37 @@ static struct fuse *fuse_instance;
 
 static void usage(const char *progname)
 {
+    if (progname)
+        fprintf(stderr,
+                "usage: %s mountpoint [FUSE options]\n\n", progname);
+
     fprintf(stderr,
-            "usage: %s mountpoint [options]\n"
-            "Options:\n"
-            "    -d                  enable debug output (implies -f)\n"
-            "    -f                  foreground operation\n"
-            "    -s                  disable multithreaded operation\n"
-            "    -o opt,[opt...]     mount options\n"
-            "    -h                  print help\n"
+            "FUSE options:\n"
+            "    -d                     enable debug output (implies -f)\n"
+            "    -f                     foreground operation\n"
+            "    -s                     disable multithreaded operation\n"
+            "    -r                     mount read only (equivalent to '-o ro')\n"
+            "    -o opt,[opt...]        mount options\n"
+            "    -h                     print help\n"
             "\n"
             "Mount options:\n"
             "    default_permissions    enable permission checking\n"
             "    allow_other            allow access to other users\n"
+            "    allow_root             allow access to root\n"
             "    kernel_cache           cache files in kernel\n"
             "    large_read             issue large read requests (2.4 only)\n"
             "    direct_io              use direct I/O\n"
             "    max_read=N             set maximum size of read requests\n"
             "    hard_remove            immediate removal (don't hide files)\n"
             "    debug                  enable debug output\n"
-            "    fsname=NAME            set filesystem name in mtab\n",
-            progname);
+            "    fsname=NAME            set filesystem name in mtab\n"
+            "    use_ino                let filesystem set inode numbers\n");
 }
 
 static void invalid_option(const char *argv[], int argctr)
 {
     fprintf(stderr, "fuse: invalid option: %s\n\n", argv[argctr]);
-    usage(argv[0]);
+    fprintf(stderr, "see `%s -h' for usage\n", argv[0]);
 }
 
 static void exit_handler()
@@ -67,7 +73,7 @@ static int set_one_signal_handler(int signal, void (*handler)(int))
         perror("FUSE: cannot get old signal handler");
         return -1;
     }
-        
+
     if (old_sa.sa_handler == SIG_DFL &&
         sigaction(signal, &sa, NULL) == -1) {
         perror("Cannot set signal handler");
@@ -87,6 +93,19 @@ static int set_signal_handlers()
     return 0;
 }
 
+static int opt_member(const char *opts, const char *opt)
+{
+    const char *e, *s = opts;
+    int optlen = strlen(opt);
+    for (s = opts; s; s = e + 1) {
+        if(!(e = strchr(s, ',')))
+            break;
+        if (e - s == optlen && strncmp(s, opt, optlen) == 0)
+            return 1;
+    }
+    return (s && strcmp(s, opt) == 0);
+}
+
 static int add_option_to(const char *opt, char **optp)
 {
     unsigned len = strlen(opt);
@@ -140,7 +159,7 @@ static int fuse_parse_cmdline(int argc, const char *argv[], char **kernel_opts,
     int argctr;
     const char *basename;
     char *fsname_opt;
-    
+
     *kernel_opts = NULL;
     *lib_opts = NULL;
     *mountpoint = NULL;
@@ -163,15 +182,15 @@ static int fuse_parse_cmdline(int argc, const char *argv[], char **kernel_opts,
     free(fsname_opt);
     if (res == -1)
         goto err;
-    
+
     for (argctr = 1; argctr < argc; argctr ++) {
         if (argv[argctr][0] == '-') {
             if (strlen(argv[argctr]) == 2)
                 switch (argv[argctr][1]) {
                 case 'o':
                     if (argctr + 1 == argc || argv[argctr+1][0] == '-') {
-                        fprintf(stderr, "missing option after -o\n\n");
-                        usage(argv[0]);
+                        fprintf(stderr, "missing option after -o\n");
+                        fprintf(stderr, "see `%s -h' for usage\n", argv[0]);
                         goto err;
                     }
                     argctr ++;
@@ -179,14 +198,19 @@ static int fuse_parse_cmdline(int argc, const char *argv[], char **kernel_opts,
                     if (res == -1)
                         goto err;
                     break;
-                    
+
                 case 'd':
                     res = add_options(lib_opts, kernel_opts, "debug");
                     if (res == -1)
                         goto err;
-                    *background = 0;
                     break;
-                    
+
+                case 'r':
+                    res = add_options(lib_opts, kernel_opts, "ro");
+                    if (res == -1)
+                        goto err;
+                    break;
+
                 case 'f':
                     *background = 0;
                     break;
@@ -194,11 +218,11 @@ static int fuse_parse_cmdline(int argc, const char *argv[], char **kernel_opts,
                 case 's':
                     *multithreaded = 0;
                     break;
-                    
+
                 case 'h':
                     usage(argv[0]);
                     goto err;
-                    
+
                 default:
                     invalid_option(argv, argctr);
                     goto err;
@@ -208,8 +232,10 @@ static int fuse_parse_cmdline(int argc, const char *argv[], char **kernel_opts,
                     res = add_options(lib_opts, kernel_opts, &argv[argctr][2]);
                     if (res == -1)
                         goto err;
-                }
-                else {
+                } else if(strcmp(argv[argctr], "-ho") == 0) {
+                    usage(NULL);
+                    goto err;
+                } else {
                     invalid_option(argv, argctr);
                     goto err;
                 }
@@ -228,8 +254,8 @@ static int fuse_parse_cmdline(int argc, const char *argv[], char **kernel_opts,
     }
 
     if (*mountpoint == NULL) {
-        fprintf(stderr, "missing mountpoint\n\n");
-        usage(argv[0]);
+        fprintf(stderr, "missing mountpoint\n");
+        fprintf(stderr, "see `%s -h' for usage\n", argv[0]);
         goto err;
     }
     return 0;
@@ -240,20 +266,23 @@ static int fuse_parse_cmdline(int argc, const char *argv[], char **kernel_opts,
     free(*mountpoint);
     return -1;
 }
-                              
 
-struct fuse *__fuse_setup(int argc, char *argv[],
-                          const struct fuse_operations *op,
-                          char **mountpoint, int *multithreaded, int *fd)
+static struct fuse *fuse_setup_common(int argc, char *argv[],
+                                      const struct fuse_operations *op,
+                                      size_t op_size,
+                                      char **mountpoint,
+                                      int *multithreaded,
+                                      int *fd,
+                                      int compat)
 {
     struct fuse *fuse;
     int background;
     char *kernel_opts;
     char *lib_opts;
     int res;
-    
+
     if (fuse_instance != NULL) {
-        fprintf(stderr, "fuse: __fuse_setup() called twice\n");
+        fprintf(stderr, "fuse: fuse_setup() called twice\n");
         return NULL;
     }
 
@@ -267,11 +296,11 @@ struct fuse *__fuse_setup(int argc, char *argv[],
     if (*fd == -1)
         goto err_free;
 
-    fuse = fuse_new(*fd, lib_opts, op);
+    fuse = fuse_new_common(*fd, lib_opts, op, op_size, compat);
     if (fuse == NULL)
         goto err_unmount;
 
-    if (background) {
+    if (background && !opt_member(lib_opts, "debug")) {
         res = daemon(0, 0);
         if (res == -1) {
             perror("fuse: failed to daemonize program\n");
@@ -299,10 +328,29 @@ struct fuse *__fuse_setup(int argc, char *argv[],
     return NULL;
 }
 
-void __fuse_teardown(struct fuse *fuse, int fd, char *mountpoint)
+struct fuse *fuse_setup(int argc, char *argv[],
+                          const struct fuse_operations *op,
+                          size_t op_size, char **mountpoint,
+                          int *multithreaded, int *fd)
+{
+    return fuse_setup_common(argc, argv, op, op_size, mountpoint,
+                             multithreaded, fd, 0);
+}
+
+struct fuse *fuse_setup_compat2(int argc, char *argv[],
+                                 const struct fuse_operations_compat2 *op,
+                                 char **mountpoint, int *multithreaded,
+                                 int *fd)
+{
+    return fuse_setup_common(argc, argv, (struct fuse_operations *) op,
+                             sizeof(struct fuse_operations_compat2),
+                             mountpoint, multithreaded, fd, 21);
+}
+
+void fuse_teardown(struct fuse *fuse, int fd, char *mountpoint)
 {
     if (fuse_instance != fuse)
-        fprintf(stderr, "fuse: __fuse_teardown() with unknown fuse object\n");
+        fprintf(stderr, "fuse: fuse_teardown() with unknown fuse object\n");
     else
         fuse_instance = NULL;
 
@@ -312,8 +360,9 @@ void __fuse_teardown(struct fuse *fuse, int fd, char *mountpoint)
     free(mountpoint);
 }
 
-
-int fuse_main(int argc, char *argv[], const struct fuse_operations *op)
+static int fuse_main_common(int argc, char *argv[],
+                            const struct fuse_operations *op, size_t op_size,
+                            int compat)
 {
     struct fuse *fuse;
     char *mountpoint;
@@ -321,19 +370,50 @@ int fuse_main(int argc, char *argv[], const struct fuse_operations *op)
     int res;
     int fd;
 
-    fuse = __fuse_setup(argc, argv, op, &mountpoint, &multithreaded, &fd);
+    fuse = fuse_setup_common(argc, argv, op, op_size, &mountpoint,
+                             &multithreaded, &fd, compat);
     if (fuse == NULL)
         return 1;
-    
+
     if (multithreaded)
         res = fuse_loop_mt(fuse);
     else
         res = fuse_loop(fuse);
-    
-    __fuse_teardown(fuse, fd, mountpoint);
+
+    fuse_teardown(fuse, fd, mountpoint);
     if (res == -1)
         return 1;
-    
+
     return 0;
 }
 
+int fuse_main_real(int argc, char *argv[], const struct fuse_operations *op,
+                   size_t op_size)
+{
+    return fuse_main_common(argc, argv, op, op_size, 0);
+}
+
+#undef fuse_main
+int fuse_main()
+{
+    fprintf(stderr, "fuse_main(): This function does not exist\n");
+    return -1;
+}
+
+void fuse_main_compat1(int argc, char *argv[],
+                      const struct fuse_operations_compat1 *op)
+{
+    fuse_main_common(argc, argv, (struct fuse_operations *) op,
+                     sizeof(struct fuse_operations_compat1), 11);
+}
+
+int fuse_main_compat2(int argc, char *argv[],
+                      const struct fuse_operations_compat2 *op)
+{
+    return fuse_main_common(argc, argv, (struct fuse_operations *) op,
+                            sizeof(struct fuse_operations_compat2), 21);
+}
+
+__asm__(".symver fuse_setup_compat2,__fuse_setup@");
+__asm__(".symver fuse_teardown,__fuse_teardown@");
+__asm__(".symver fuse_main_compat2,fuse_main@");
