@@ -1,12 +1,13 @@
 /*
     FUSE: Filesystem in Userspace
-    Copyright (C) 2001  Miklos Szeredi (mszeredi@inf.bme.hu)
+    Copyright (C) 2001-2005  Miklos Szeredi <miklos@szeredi.hu>
 
     This program can be distributed under the terms of the GNU LGPL.
     See the file COPYING.LIB.
 */
 
 #include "fuse.h"
+#include "fuse_compat.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -56,7 +57,7 @@ static int receive_fd(int fd)
         /* EOF */
         return -1;
     }
-    
+
     cmsg = CMSG_FIRSTHDR(&msg);
     if (!cmsg->cmsg_type == SCM_RIGHTS) {
         fprintf(stderr, "got control message of unknown type %d\n",
@@ -70,15 +71,15 @@ void fuse_unmount(const char *mountpoint)
 {
     const char *mountprog = FUSERMOUNT_PROG;
     char umount_cmd[1024];
-    
-    snprintf(umount_cmd, sizeof(umount_cmd) - 1, "%s -u -q %s", mountprog,
+
+    snprintf(umount_cmd, sizeof(umount_cmd) - 1, "%s -u -q -z %s", mountprog,
              mountpoint);
-    
+
     umount_cmd[sizeof(umount_cmd) - 1] = '\0';
     system(umount_cmd);
 }
 
-int fuse_mount(const char *mountpoint, const char *args[])
+int fuse_mount(const char *mountpoint, const char *opts)
 {
     const char *mountprog = FUSERMOUNT_PROG;
     int fds[2], pid;
@@ -91,7 +92,11 @@ int fuse_mount(const char *mountpoint, const char *args[])
         return -1;
     }
 
+#ifndef USE_UCLIBC
     pid = fork();
+#else
+    pid = vfork();
+#endif
     if(pid == -1) {
         perror("fuse: fork() failed");
         close(fds[0]);
@@ -101,28 +106,14 @@ int fuse_mount(const char *mountpoint, const char *args[])
 
     if(pid == 0) {
         char env[10];
-        char **newargv;
-        int numargs = 0;
-        int actr;
-        int i;
-
-        if(args != NULL) 
-            while(args[numargs] != NULL)
-                numargs ++;
-
-        newargv = (char **) malloc((1 + numargs + 2) * sizeof(char *));
-        actr = 0;
-        newargv[actr++] = strdup(mountprog);
-        for(i = 0; i < numargs; i++)
-            newargv[actr++] = strdup(args[i]);
-        newargv[actr++] = strdup(mountpoint);
-        newargv[actr++] = NULL;
+        const char *argv[] = {mountprog, opts ? "-o" : mountpoint, opts,
+                              mountpoint, NULL};
 
         close(fds[1]);
         fcntl(fds[0], F_SETFD, 0);
         snprintf(env, sizeof(env), "%i", fds[0]);
         setenv(FUSE_COMMFD_ENV, env, 1);
-        execvp(mountprog, newargv);
+        execvp(mountprog, (char **) argv);
         perror("fuse: failed to exec fusermount");
         exit(1);
     }
@@ -134,3 +125,10 @@ int fuse_mount(const char *mountpoint, const char *args[])
 
     return rv;
 }
+
+int fuse_mount_compat1(const char *mountpoint, const char *args[])
+{
+    /* just ignore mount args for now */
+    (void) args;
+    return fuse_mount(mountpoint, NULL);
+}
